//==================================================================================================
// Name        : caLab.h
// Author      : Tobias Hft, Carsten Winkler
// Version     : 1.0.0.2
// Copyright   : HZB
// Description : library for get, put and monitoring EPICS variables (process variables) in LabVIEW
//==================================================================================================

#include <vector>
#include <string>

#include <cadef.h>
#include <epicsMutex.h>
#include <envDefs.h>

#ifndef CAMONITOR_H_
#define CAMONITOR_H_

#define EXPORT __declspec(dllexport)
#define MAX_ERROR_SIZE 255

#pragma pack (1)

typedef char EpicsStr[MAX_STRING_SIZE]; // valid EPICS string

unsigned int dllLoad;
bool isRunning = false; // indicator for valid channel access context
epicsMutexId mutex;
const double REFRESH_RATE = 0.1; // to avoid another execution of caget in that time interval (sec)

typedef struct {
	char name[MAX_STRING_SIZE]; // MAX_STRING_SIZE defined in epicsTypes.h
	chid chanid;
	evid eventid;
	evid eventidEnum;
	chtype dbrType;
	unsigned short enumRecord; // 0 - no enum, 1 - enum record
	unsigned short enumAsString; // 0 - as number, 1 - as string
	unsigned long elems; // number of elements of this record
	channel_state connectionStatus; // cs_never_conn, cs_prev_conn, cs_conn, cs_closed
	unsigned short monitor; // bool -> 1 - monitor should or is running; 0 - no monitor
	epicsTimeStamp timestamp;

	EpicsStr* sValueArr;
	double* dValueArr;
	char sStatus[MAX_STRING_SIZE];
	short dStatus;
	char sSeverity[MAX_STRING_SIZE];
	short dSeverity;
	char sTime[MAX_STRING_SIZE];
	int dTime;
	char sError[MAX_ERROR_SIZE];
	short dError;

	chid chanidEgu;
	chid chanidDesc;
	evid eventidEgu;
	evid eventidDesc;
	char unit[MAX_UNITS_SIZE];
	char desc[MAX_STRING_SIZE];

	epicsThreadId threadid;

	void* putArr; // array to put if connection is down (connection handler puts array)
	unsigned long elemsToPut; // number of elements to put
} Admin;

std::vector<Admin*> admin;

// callback for connection events
void connectionChanged(struct connection_handler_args args);
// callback for value events
void valueChanged(struct event_handler_args args);
// callback for connection events

#ifdef __cplusplus
extern "C" {
#endif

EXPORT int initalizeVariable(char sError[MAX_ERROR_SIZE], char pvname[MAX_STRING_SIZE], long* id);
EXPORT int disconnectVariable(char sError[MAX_ERROR_SIZE], long* id);
EXPORT int diconnectAllVariables(char sError[MAX_ERROR_SIZE]);
EXPORT int addVariableToMonitor(char sError[MAX_ERROR_SIZE], long* id);
EXPORT int removeVariableFromMonitor(char sError[MAX_ERROR_SIZE], long* id);
EXPORT int readMonitor(char sError[MAX_ERROR_SIZE], long* id, double* dArray, char sStatus[MAX_STRING_SIZE], short* dStatus, char sSeverity[MAX_STRING_SIZE], short* dSeverity, char sTime[MAX_STRING_SIZE], int* dTime, char* str);
EXPORT int caGet(char sError[MAX_ERROR_SIZE], long* id, double* dArray, char sStatus[MAX_STRING_SIZE], short* dStatus, char sSeverity[MAX_STRING_SIZE], short* dSeverity, char sTime[MAX_STRING_SIZE], int* dTime, char* str);
EXPORT int caPut(char sError[MAX_ERROR_SIZE], long* id, unsigned long* elems, unsigned short* enumAsString, void* dArray, char* str);

EXPORT int getElemsAndType(char sError[MAX_ERROR_SIZE], long* id, unsigned long* elems, char dbrType[MAX_ERROR_SIZE]);
EXPORT int buildOverviewArray(char sError[MAX_ERROR_SIZE], char pvname[MAX_STRING_SIZE], long* id, char unit[MAX_UNITS_SIZE], char desc[MAX_STRING_SIZE], short* status, char dbrType[MAX_STRING_SIZE], unsigned short* count, unsigned short* end);

EXPORT int getEgu(char sError[MAX_ERROR_SIZE], long* id);
EXPORT int getDesc(char sError[MAX_ERROR_SIZE], long* id);

EXPORT void preload();

#ifdef __cplusplus
}
#endif

#endif /* CAMONITOR_H_ */
